#' Region navn og region kode for gitt tidspunkt
#'
#' Denne funksjonen tar regionkoder og regionnavn fra en regional statistikk,
#' tar inn tabell som viser historiske endringer i regional inndeling av Norge
#' og gjør om til riktige koder og navn for et gitt referanseår (ref.yr)
#' Funksjonen fungerer for fylkesnivå inkludert landet ELLER for kommunenivå.
#'
#' @param regionstat is the regional statistics, where each obs represent one yar
#' and one region (fylke eller kommune)
#' @param ref.yr is the reference year to which geographic name and code is to be used
#'
#' @return tibble having the regional statistics including the regional
#' names and codes for the reference year in question
#' @export
#' @importFrom rlang :=
#' @importFrom dplyr mutate
#'
#'
#' @examples regnavn.at.ref.yr(regionstat = t12750()) %>% glimpse()
regnavn.at.ref.yr <- function(regionstat, ref.yr = lubridate::year(lubridate::now())){
  #    regionstat = t12750() #for testing

  # Fetch the relevant region reference table regref
  if (nchar(regionstat$region_kode[1]) == 2) {
    regref <- vsop::regref_fylke_l #regref_fylke_l is a table created in the DATASET script saved in the "data" directory
  } else {
    regref <- vsop::regref_kommune_l #regref_kommune_l is a table created in the DATASET script saved in the "data" directory
  }

  # harmonizing
  ref.yr <- as.integer(ref.yr)


  # identify which row in regref the region_code and observation year belongs to:


 regionstat <-
    regionstat %>%
    dplyr::mutate(
                  # which row in regref the region_code and observation year belongs to:
                  regrefrow = purrr::pmap_int(regionstat, .f = function(region_kode, aar, ...){
                    # 1: check if exists a part of regref where "region_code_from" == region_kode AND yfrom <= aar
                    tmprr <-
                      dplyr::filter(regref, reg_code_from == region_kode, yfrom <= aar)

                    if (nrow(tmprr) > 0 ) {
                      # yes then tag which row in regref the region_code and observation year belongs to. regref reference row
                      rrr <-  max( which(regref$reg_code_from  == region_kode & regref$yfrom <= aar))
                    } else {
                      rrr <- NA_integer_
                    }
                    return(rrr)
                  }))  %>%
   dplyr::filter( !is.na(regrefrow)) %>%


    #   Then we have index needed to pick the right row
    #      to populate both reg_k@ref.yr and reg_n@ref.yr
    dplyr::mutate(
                  !!rlang::sym(paste0("reg_n", ref.yr)) := regref$reg_name_to[(regrefrow)],
                  !!rlang::sym(paste0("reg_k", ref.yr)) := regref$reg_code_to[(regrefrow)]
    )


  return(regionstat)

}







